/*
 * This file is a part of VyQChat.
 *
 * Copyright (C) 2002-2004 Pawel Stolowski <yogin@linux.bydg.org>
 *
 * VyQChat is free software; you can redestribute it and/or modify it
 * under terms of GNU General Public License by Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY. See GPL for more details.
 */

#include "global.h"
#include "settings.h"

#include <unistd.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <qsettings.h>

#ifndef DATADIR
#define DATADIR "./"
#endif

Settings::Settings()/*{{{*/
{
}/*}}}*/

Settings::~Settings()/*{{{*/
{
}/*}}}*/

bool Settings::getDefaults()/*{{{*/
{
	QString datapath(DATADIR);

	bool status = false;
	def_topic = "no topic";
	def_sound = Sound::PCSpeaker;
	def_playcmd = "/usr/bin/play";
	def_snd[Sound::Chatline] = datapath + "chatline.wav";
	def_snd[Sound::Beep] = datapath + "beep.wav";
	def_snd[Sound::JoinChannel] = datapath + "join.wav";
	def_snd[Sound::LeaveChannel] = datapath + "leave.wav";
	def_snd[Sound::Message] = datapath + "beep.wav";
	def_snd[Sound::RegexpMatch] = datapath + "beep.wav";

	//
	// get hostname
	char tmp[255];
	gethostname(tmp, 254);
	def_hostname = QString(tmp);

	//
	// get username
	char *login = getenv("LOGNAME");
	def_nick = QString(login);

	//
	// get eth0 parameters
	def_broadcast = "127.255.255.255";
	def_ip = "127.0.0.1";

	struct ifconf ifc;
	int sockfd = socket(AF_INET, SOCK_DGRAM, 0);

	ifc.ifc_req = new ifreq [16];
	ifc.ifc_len = 16*sizeof(struct ifreq);

	if (ioctl(sockfd, SIOCGIFCONF, &ifc)==0)
	{
		status = true;

		const int num = ifc.ifc_len/sizeof(struct ifreq);
		struct ifreq *req = ifc.ifc_req; 
		for (int i=0; i<num; i++)
		{
			if (strcmp(req[i].ifr_name, "eth0")==0)
			{
				if (ioctl(sockfd, SIOCGIFBRDADDR , &req[i])==0) 
					def_broadcast = QString(inet_ntoa(*(in_addr *)(&req[i].ifr_broadaddr.sa_data[2])));
				else
					status = false;
				
				if (ioctl(sockfd, SIOCGIFADDR , &req[i])==0) 
					def_ip = QString(inet_ntoa(*(in_addr *)(&req[i].ifr_addr.sa_data[2])));
				else
					status = false;
			}
		}
		
	}
	delete [] ifc.ifc_req;
	close(sockfd);
	return status;
}/*}}}*/

bool Settings::load(QSettings *cfg)/*{{{*/
{
	Q_ASSERT(cfg);

	getDefaults();

	cfg->beginGroup("/Network");
		nick = cfg->readEntry("/Nick", def_nick);
		hostname = cfg->readEntry("/Hostname", def_hostname);
		ip = cfg->readEntry("/IP", def_ip);
		broadcast = cfg->readEntry("/Broadcast", def_broadcast);
		refresh = cfg->readNumEntry("/Refresh", 300);
	cfg->endGroup();
	cfg->beginGroup("/Misc");
		encoding = cfg->readEntry("/Encoding", "");
		hide_channels = cfg->readBoolEntry("/HideChannels", false);
		popup = cfg->readBoolEntry("/Popup", true);
		nopopup_dnd = cfg->readBoolEntry("/NoPopupDND", true);
	cfg->endGroup();
	cfg->beginGroup("/General");
		version = cfg->readEntry("/Version", "0");
		regexp_on = cfg->readBoolEntry("/RegExpOn", false);
		toolbar_on = cfg->readBoolEntry("/ToolbarOn", true);
		userslist_on = cfg->readBoolEntry("/UsersListOn", true);
		nickcombo_on = cfg->readBoolEntry("/NickComboOn", true);
		confirm_exit = cfg->readBoolEntry("/ConfirmExit", true);
		topic = cfg->readEntry("/Topic", def_topic);
		keep_status = cfg->readBoolEntry("/KeepStatus", true);
		status = cfg->readNumEntry("/Status", STATUS_NORMAL);
	cfg->endGroup();
	cfg->beginGroup("/Sound");
		sound = static_cast<Sound::SoundType>(cfg->readNumEntry("/sound", def_sound));
		playcmd = cfg->readEntry("/PlayCommand", def_playcmd);
	cfg->endGroup();
	cfg->beginGroup("/Wavs");
		snd[Sound::Chatline] = cfg->readEntry("/Chatline", def_snd[Sound::Chatline]);
		snd[Sound::Beep] = cfg->readEntry("/Beep", def_snd[Sound::Beep]);
		snd[Sound::Message] = cfg->readEntry("/Message", def_snd[Sound::Message]);
		snd[Sound::JoinChannel] = cfg->readEntry("/Join", def_snd[Sound::JoinChannel]);
		snd[Sound::LeaveChannel] = cfg->readEntry("/Leave", def_snd[Sound::LeaveChannel]);
		snd[Sound::RegexpMatch] = cfg->readEntry("/RegexpMatch", def_snd[Sound::RegexpMatch]);
		sndp[Sound::Chatline] = cfg->readBoolEntry("/pChatline", false);
		sndp[Sound::Beep] = cfg->readBoolEntry("/pBeep", false);
		sndp[Sound::Message] = cfg->readBoolEntry("/pMessage", false);
		sndp[Sound::JoinChannel] = cfg->readBoolEntry("/pJoin", false);
		sndp[Sound::LeaveChannel] = cfg->readBoolEntry("/pLeave", false);
		sndp[Sound::RegexpMatch] = cfg->readBoolEntry("/pRegexpMatch", false);
	cfg->endGroup();
	cfg->beginGroup("/Window");
		x = cfg->readNumEntry("/x", -1);
		y = cfg->readNumEntry("/y", -1);
		width = cfg->readNumEntry("/Width", 480);
		height = cfg->readNumEntry("/Height", 360);
		listwidth = cfg->readNumEntry("/ListWidth", 120);
	cfg->endGroup();
	return true;
}/*}}}*/

bool Settings::save(QSettings *cfg)/*{{{*/
{	
	Q_ASSERT(cfg);
	
	//cfg->beginGroup("/vyqchat");

	cfg->beginGroup("/Network");
		cfg->writeEntry("/Nick", nick);
		cfg->writeEntry("/Hostname", hostname);
		cfg->writeEntry("/IP", ip);
		cfg->writeEntry("/Broadcast", broadcast);
		cfg->writeEntry("/Refresh", refresh);
	cfg->endGroup();
	cfg->beginGroup("/Misc");
		cfg->writeEntry("/Encoding", encoding);
		cfg->writeEntry("/HideChannels", hide_channels);
		cfg->writeEntry("/Popup", popup);
		cfg->writeEntry("/NoPopupDND", nopopup_dnd);
	cfg->endGroup();
	cfg->beginGroup("/General");
		cfg->writeEntry("/Version", VYQCHAT_VERSION);
		cfg->writeEntry("/RegExpOn", regexp_on);
		cfg->writeEntry("/ToolbarOn", toolbar_on);
		cfg->writeEntry("/UsersListOn", userslist_on);
		cfg->writeEntry("/NickComboOn", nickcombo_on);
		cfg->writeEntry("/Topic", topic);
		cfg->writeEntry("/ConfirmExit", confirm_exit);
		cfg->writeEntry("/KeepStatus", keep_status);
		cfg->writeEntry("/Status", status);
	cfg->endGroup();
	cfg->beginGroup("/Sound");
		cfg->writeEntry("/sound", static_cast<int>(sound));
		cfg->writeEntry("/PlayCommand", playcmd);
	cfg->endGroup();
	cfg->beginGroup("/Wavs");
		cfg->writeEntry("/Chatline", snd[Sound::Chatline]);
		cfg->writeEntry("/Join", snd[Sound::JoinChannel]);
		cfg->writeEntry("/Leave", snd[Sound::LeaveChannel]);
		cfg->writeEntry("/Beep", snd[Sound::Beep]);
		cfg->writeEntry("/Message", snd[Sound::Message]);
		cfg->writeEntry("/RegexpMatch", snd[Sound::RegexpMatch]);
		cfg->writeEntry("/pChatline", sndp[Sound::Chatline]);
		cfg->writeEntry("/pJoin", sndp[Sound::JoinChannel]);
		cfg->writeEntry("/pLeave", sndp[Sound::LeaveChannel]);
		cfg->writeEntry("/pBeep", sndp[Sound::Beep]);
		cfg->writeEntry("/pMessage", sndp[Sound::Message]);
		cfg->writeEntry("/pRegexpMatch", sndp[Sound::RegexpMatch]);
	cfg->endGroup();
	cfg->beginGroup("/Window");
		cfg->writeEntry("/x", x);
		cfg->writeEntry("/y", y);
		cfg->writeEntry("/Width", width);
		cfg->writeEntry("/Height", height);
		cfg->writeEntry("/ListWidth", listwidth);
	cfg->endGroup();
	//cfg->endGroup();
	return true;
}/*}}}*/

