/*
 * This file is a part of VyQChat.
 *
 * Copyright (C) 2002-2004 Pawel Stolowski <yogin@linux.bydg.org>
 *
 * VyQChat is free software; you can redestribute it and/or modify it
 * under terms of GNU General Public License by Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY. See GPL for more details.
 */

#include "vypacket.h"
#include <qglobal.h>

VyPacket::VyPacket(const QHostAddress &src, char *p_data, int p_len, QTextCodec *c):/*{{{*/
	channel(NULL), from(NULL), to(NULL), status(0), gender(0), text(NULL), source(src)
{
	Q_ASSERT(p_data!=NULL);
	decode(p_data, p_len, c);
}/*}}}*/

VyPacket::~VyPacket()/*{{{*/
{
}/*}}}*/

bool VyPacket::split(char *buffer, int num, int p_len)/*{{{*/
{
	const char *bend = buffer + p_len; //buffer end pointer
	for (int i=0; i<num; i++)
	{
		ptrs[i] = buffer; //store pointer to the beginning of substring
		for (; (buffer<bend) && (*buffer); buffer++); //move to the end of this substring
		if (buffer >= bend)
			return false;
		buffer++;                //skip zero byte
	}
	return true;
}/*}}}*/

bool VyPacket::decode(char *p_data, int p_len, QTextCodec *c)/*{{{*/
{
	char *p = p_data;
	if (*p != 'X')
		return false;
	p += 10; 		//skip packet id
	char b = *(p++);

	switch (b)
	{
		/* WHO HERE
	 	* updater */
		case VY_WHOHERE:
			from = c->toUnicode(p);
			ptype = VyWhoHere;
			break;
		/* IM HERE
		 * updater :: responder :: status + remote active */
		case VY_IMHERE:
			split(p, 3, p_len);
			to = c->toUnicode(ptrs[0]);
			from = c->toUnicode(ptrs[1]);
			status = ptrs[2][0];
			remoteactive = ptrs[2][1];
			ptype = VyImHere;
			break;
		/* SIMPLE CHAT
		 * channel :: author :: text */
		case VY_SAY: 
			split(p, 3, p_len);
			from = c->toUnicode(ptrs[1]);
			channel = c->toUnicode(ptrs[0]);
			text = c->toUnicode(ptrs[2]);
			ptype = VySay;
			break;
		/* ME
		 * channel :: author :: text */
		case VY_ME:
			split(p, 3, p_len);
			channel = c->toUnicode(ptrs[0]);
			from = c->toUnicode(ptrs[1]);
			text = c->toUnicode(ptrs[2]);
			ptype = VyMe;
			break;
		/* NICK CHANGE
		 * old :: new :: gender */
		case VY_NICKCHANGE: 
			split(p, 3, p_len);
			from = c->toUnicode(ptrs[0]);
			to = c->toUnicode(ptrs[1]);
			gender = ptrs[2][0];
			ptype = VyNickChange;
			break;
		case VY_BEEP:
			  b = *(p++);
			  switch (b)
			  {
				  /* BEEP SIGNAL
				   * to :: from  */
				  case VY_BEEP_SIGNAL: 
					  split(p, 2, p_len);
					  to = c->toUnicode(ptrs[0]);
					  from = c->toUnicode(ptrs[1]);
					  ptype = VyBeepSignal;
					  break;
				  /* BEEP ACK
				   * to :: from :: gender */
				  case VY_BEEP_ACK: 
					  split(p, 3, p_len);
					  to = c->toUnicode(ptrs[0]);
					  from = c->toUnicode(ptrs[1]);
					  gender = ptrs[2][0];
					  ptype = VyBeepAck;
					  break;
				  default: break;
			  }
			  break;
		/* JOIN
		 * who :: channel :: status + gender */
		case VY_JOIN: 
			  split(p, 3, p_len);
			  from = c->toUnicode(ptrs[0]);
			  channel = c->toUnicode(ptrs[1]);
			  status = ptrs[2][0];
			  gender = ptrs[2][1];
			  ptype = VyJoin;
			  break;
		/* LEAVE
		 * who :: channel :: gender */
		case VY_LEAVE:
			  split(p, 3, p_len);
			  from = c->toUnicode(ptrs[0]);
			  channel = c->toUnicode(ptrs[1]);
			  gender = ptrs[2][0];
			  ptype = VyLeave;
			  break;	  
		/* PRIVATE */
		case VY_PRIVATE:
			  b = *(p++);
			  split(p, 3, p_len);
			  from = c->toUnicode(ptrs[0]);
			  to = c->toUnicode(ptrs[1]);
			  switch (b)
			  {
				  /* PRIVATE JOIN, PRIVATE_LEAVE
				   * from :: to :: gender */
				  case VY_PRIVATE_JOIN:	
					  	gender = ptrs[2][0];
					  	ptype = VyPrivateJoin;
						break;
				  case VY_PRIVATE_LEAVE:
					  	gender = ptrs[2][0];
						ptype = VyPrivateLeave;
						break;
				  /* PRIVATE_SAY, PRIVATE_ME
				   * from :: to :: text */
				  case VY_PRIVATE_SAY:
						text = c->toUnicode(ptrs[2]);
						ptype = VyPrivateSay;
						break;
				  case VY_PRIVATE_ME:  
						text = c->toUnicode(ptrs[2]);
						ptype = VyPrivateMe;
						break;
				  default: break;
			  }
			  break;
		/* MESSAGE
		 * from :: to :: text */
		case VY_MESSAGE: 
			  split(p, 3, p_len);
			  from = c->toUnicode(ptrs[0]);
			  to = c->toUnicode(ptrs[1]);
			  text = c->toUnicode(ptrs[2]);
			  ptype = VyMessage;
			  break;
		/* MESSAGEACK
		 * status + whorec :: towhoack :: gender + currentaa */	  
		case VY_MESSAGEACK:
			  split(p, 3, p_len);
			  status = ptrs[0][0];
			  from = ptrs[1];
			  to = c->toUnicode(ptrs[0]+1);
			  gender = ptrs[2][0];
			  remoteactive = ptrs[2][1];
			  ptype = VyMessageAck;
			  break;
		/* MASSMESSAGE
		 * from :: to :: text */
		case VY_MASSMESSAGE:
			  split(p, 3, p_len);
			  from = ptrs[0];
			  to = ptrs[1];
			  text = ptrs[2];
			  ptype = VyMassMessage;
			  break;
		/* HERE REQUEST
		 * from :: channel */
		case VY_HEREREQ:
			  split(p, 2, p_len);
			  from = c->toUnicode(ptrs[0]);
			  channel = c->toUnicode(ptrs[1]);
			  ptype = VyHereReq;
			  break;
		/* HERE ACK 
		 * to :: channel :: from :: remote active */
		case VY_HEREACK:
			  split(p, 4, p_len);
			  to = c->toUnicode(ptrs[0]);
			  channel = c->toUnicode(ptrs[1]);
			  from = c->toUnicode(ptrs[2]);
			  remoteactive = ptrs[3][0];
			  ptype = VyHereReqAck;
			  break;
		/* INFO REQUEST
		 * to :: from */
		case VY_INFOREQ: 
			  split(p, 2, p_len);
			  to = c->toUnicode(ptrs[0]);
			  from = c->toUnicode(ptrs[1]);
			  ptype = VyInfoReq;
			  break;
		/* INFO REQUEST ACK
		 * to :: from :: computer :: name :: ip :: channels :: answer */
		case VY_INFOREQACK:  
			  split(p, 7, p_len);
			  to = c->toUnicode(ptrs[0]);
			  from = c->toUnicode(ptrs[1]);
			  computer = c->toUnicode(ptrs[2]);
			  name = c->toUnicode(ptrs[3]);
			  ip = c->toUnicode(ptrs[4]);
			  channel = c->toUnicode(ptrs[5]);
			  text = c->toUnicode(ptrs[6]);
			  ptype = VyInfoReqAck;
			  break;
	 	/* CURRENT TOPIC
		 * towho :: channel :: topic */
		case VY_CURRTOPIC: 
			  split(p, 3, p_len);
			  to = c->toUnicode(ptrs[0]);
			  channel = c->toUnicode(ptrs[1]);
			  topic = c->toUnicode(ptrs[2]);
			  ptype = VyCurrTopic;
			  break;
		/* NEW TOPIC
		 * channel :: topic */
		case VY_NEWTOPIC:
			  split(p, 2, p_len);
			  channel = c->toUnicode(ptrs[0]);
			  topic = c->toUnicode(ptrs[1]);
			  ptype = VyNewTopic;
			  break;
		/* CHANGE STATUS
		 * who :: status + gender + aa */
		case VY_CHANGESTAT:
			  split(p, 2, p_len);
			  from = c->toUnicode(ptrs[0]);
			  status = ptrs[1][0];
			  gender = ptrs[1][1];
			  text = c->toUnicode(ptrs[1]+2);
			  ptype = VyChangeStat;
			  break;
		default:  ptype = VyUnknown;
			  qWarning("packet %c not implemented (yet)", b);
			  return false;
			break;
	}
	return true;
}/*}}}*/

